@@ -1,10 +1,14 @@
 import numpy as np
 import torch
+import warnings
 from pdb import set_trace
 from scipy.optimize import linear_sum_assignment
 from dl_utils.tensor_funcs import numpyify
 
 
+class TranslationError(Exception):
+    pass
+
 def unique_labels(labels):
     if isinstance(labels,np.ndarray) or isinstance(labels,list):
         return set(labels)
@@ -15,7 +19,8 @@ def unique_labels(labels):
         print("Unrecognized type for labels:", type(labels))
         raise TypeError
 def label_assignment_cost(labels1,labels2,label1,label2):
-    assert len(labels1) == len(labels2), f"len labels1 {len(labels1)} must equal len labels2 {len(labels2)}"
+    if len(labels1) != len(labels2):
+        raise TranslationError(f"len labels1 {len(labels1)} must equal len labels2 {len(labels2)}")
     return len([idx for idx in range(len(labels2)) if labels1[idx]==label1 and labels2[idx] != label2])
 
 def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
@@ -24,11 +29,13 @@ def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
     trans_to_labels, tdt, _ = compress_labels(trans_to_labels)
     reverse_tdf = {v:k for k,v in tdf.items()}
     reverse_tdt = {v:k for k,v in tdt.items()}
-    assert trans_from_labels.shape == trans_to_labels.shape
+    if trans_from_labels.shape != trans_to_labels.shape:
+        raise TranslationError(f"trans_to_labels: {trans_to_labels.shape} doesn't equal trans_to_labels shape: {trans_from_labels.shape}")
     num_from_labs = get_num_labels(trans_from_labels)
     num_to_labs = get_num_labels(trans_to_labels)
     if subsample_size != 'none':
-        assert subsample_size > min(num_from_labs,num_to_labs)
+        if trans_from_labels.shape != trans_to_labels.shape:
+            raise TranslationError(f"subsample_size is too small, it must be at least min of the number of different from labels and the number of different to labels, which in this case are {num_from_labs} and {num_to_labs}")
         subsample_size = min(len(trans_from_labels),subsample_size)
     if num_from_labs <= num_to_labs:
         trans_dict = get_fanout_trans_dict(trans_from_labels,trans_to_labels,subsample_size)
@@ -40,6 +47,11 @@ def get_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
     return trans_dict,leftovers
 
 def translate_labellings(trans_from_labels,trans_to_labels,subsample_size):
+    if trans_from_labels.shape != trans_to_labels.shape:
+        raise TranslationError(f"trans_to_labels: {trans_to_labels.shape} doesn't equal trans_to_labels shape: {trans_from_labels.shape}")
+    if len(trans_from_labels) == 0:
+        warnings.warn("You're translating an empty labelling")
+        return trans_from_labels
     trans_dict, leftovers = get_trans_dict(trans_from_labels,trans_to_labels,subsample_size)
     return np.array([trans_dict[l] for l in trans_from_labels])
 
@@ -59,8 +71,8 @@ def get_fanout_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
             if unique_labels(trans_from_labels_subsample) == unique_trans_from_labels and unique_labels(trans_to_labels_subsample) == unique_trans_to_labels: break
         cost_matrix = np.array([[label_assignment_cost(trans_from_labels_subsample,trans_to_labels_subsample,l1,l2) for l2 in unique_trans_from_labels if l2 != -1] for l1 in unique_trans_from_labels if l1 != -1])
     row_ind, col_ind = linear_sum_assignment(cost_matrix)
-    try: assert len(col_ind) == len(set(trans_from_labels[trans_from_labels != -1]))
-    except: set_trace()
+    if len(col_ind) != len(set(trans_from_labels[trans_from_labels != -1])):
+        raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
     trans_dict = {l:col_ind[l] for l in unique_labels(trans_from_labels)}
     trans_dict[-1] = -1
     return trans_dict
@@ -81,7 +93,8 @@ def get_fanin_trans_dict(trans_from_labels,trans_to_labels,subsample_size):
         trans_to_labels_subsample = trans_to_labels[sample_indices]
         cost_matrix = np.array([[label_assignment_cost(trans_to_labels_subsample,trans_from_labels_subsample,l1,l2) for l2 in unique_trans_from_labels if l2 != -1] for l1 in unique_trans_to_labels if l1 != -1])
     row_ind, col_ind = linear_sum_assignment(cost_matrix)
-    assert len(col_ind) == get_num_labels(trans_to_labels)
+    if len(col_ind) != get_num_labels(trans_to_labels):
+        raise TranslationError(f"then translation cost matrix is the wrong size for some reason")
     untranslated = [i for i in range(cost_matrix.shape[1]) if i not in col_ind]
     unique_untranslated = unique_labels(untranslated)
     # Now assign the additional, unassigned items
@@ -107,7 +120,8 @@ def get_confusion_mat(labels1,labels2):
 def debable(labellings,pivot,subsample_size='none'):
     #labellings_list.sort(key=lambda x: x.max())
     if isinstance(labellings,np.ndarray):
-        assert labellings.ndim == 2, f"If debabling array, it should have 2 dimensions, but here it has {labellings.ndim}"
+        if labellings.ndim != 2:
+            raise TranslationError(f"If debabling array, it should have 2 dimensions, but here it has {labellings.ndim}")
         labellings_list = [r for r in labellings]
     else:
         labellings_list = labellings
@@ -160,3 +174,10 @@ def get_num_labels(labels):
 def label_counts(labels):
     assert labels.ndim == 1
     return {x:sum(labels==x) for x in unique_labels(labels)}
+
+def dummy_labels(num_classes,size):
+    main_chunk = np.tile(np.arange(num_classes),size//num_classes)
+    extra_chunk = np.arange(num_classes)[:size%num_classes]
+    combined = np.concatenate((main_chunk,extra_chunk), axis=0)
+    assert combined.shape[0] == size
+    return combined
